Attribute VB_Name = "WaveTime"
'Written in Visual Basic 5
'Written by Robert Orndorff in June, 1999
'orndorff@infi.net
'http://www.rmonet.com
'
'You are free to use this in any program in any way you want,
'free of charge.
'If you use this routine in your program, please
'at least mention my name.
'("Thanks to the Great and Powerful Robert Orndorff for the
'GetWaveTime routine", for example.)
'
'If you distribute this code unaltered, then keep my comments
'intact.
'
'If you alter this code then distribute it, mention that you based
'your code on my routine.
'
'If you find a bug or come up with an improvement or optimization,
'please let me know about it.

Function GetWaveTime(WaveFileName As String) As Double

On Local Error Resume Next
'Standard Wave file header info
'Note that most languages and hex viewer/editors
'consider the first byte in a file to be 0
'VB considers the first byte in a file to be 1
'
'These are usual locations, but not always!
'This code will find it regardless of where it starts
'bytes 0-3 ="RIFF" ckID1
'bytes 4-7 = (File size - 4) 32 bit
'bytes 8-11 = Type of RIFF "WAVE"
'bytes 12-15 = format = "fmt " ckID2
'bytes 16-20 = FORMAT Chunk length in bytes (should be 16)
'bytes 20-21 = format tag (1=PCM)
'bytes 22-23 = Number of channels
'bytes 24-27 = sample rate
'bytes 28-31 = avg bytes/sec - usually useless
'bytes 32-33 = block alignment - usually useless
'bytes 34-35 = bits / sample(8 Or 16)
'bytes 36-39 = Start of data chunk - ckID3 = "data"
'bytes 40-43 = number of samples
'bytes 44-EOF = Data
'Wave header info from:
'<http://www.compsoc.man.ac.uk/~maniac/resource_web/wav_file_format/wav2.htm>
'<http://www.borg.com/~jglatt/tech/wave.htm>
'Wave headers can be in variable positions but...
'the information always follows the Chunk header

Dim ckID1 As String * 4
Dim ckID2 As String * 4
Dim RIFFID As String * 4
Dim FormatTag As Integer
Dim FormatChunkLength As Long
Dim NumChannels As Integer
Dim SampleRate As Long
Dim BitsPerSample As Integer
Dim NumSamples As Long
Dim Length As Double
Dim NotWave As Boolean
Dim Offset As Integer
Dim HeaderData As String * 100
Dim WaveFile As Integer

'Get an available file number
WaveFile = FreeFile

'Check that the file can be opened
Open WaveFileName For Input As WaveFile
If Err.Number <> 0 Then
    msg$ = "Error opening file " & WaveFileName & vbCrLf
    msg$ = msg$ + Str$(Err.Number) & " - " & Err.Description
    MsgBox msg$
    Exit Function
End If

Close WaveFile

'.wav header is at least 44 bytes long
'No valid .wav file will be less than 45 bytes
If FileLen(WaveFileName) < 45 Then
    MsgBox "Invalid .WAV file!"
    Close WaveFile
    GetWaveTime = 0
    Exit Function
End If

'Now open the file for reading.
Open WaveFileName For Binary As WaveFile

Seek WaveFile, 1
Get WaveFile, , HeaderData

Seek WaveFile, 1
'Should be "RIFF"
Get WaveFile, , ckID1

Seek WaveFile, 9
'Should be "WAVE"
Get WaveFile, , RIFFID

Offset = InStr(HeaderData, "fmt ")
Seek WaveFile, Offset
'Should be "fmt "
Get WaveFile, , ckID2

Seek WaveFile, Offset + 4
Get WaveFile, , FormatChunkLength

Seek WaveFile, Offset + 8
Get WaveFile, , FormatTag

Seek WaveFile, Offset + 10
Get WaveFile, , NumChannels

Seek WaveFile, Offset + 12
Get WaveFile, , SampleRate

Seek WaveFile, Offset + 22
Get WaveFile, , BitsPerSample

Offset = InStr(HeaderData, "data")
Seek WaveFile, Offset + 4
Get WaveFile, , NumSamples

Close WaveFile

'Check for valid wave file
If ckID1 <> "RIFF" Or RIFFID <> "WAVE" Or ckID2 <> "fmt " Then
    'Not a valid wave file
    NotWave = True
    'Messages for debugging
    msg$ = "ckID =" & ckID & vbCrLf
    msg$ = msg$ + "RiffID =" & RIFFID & vbCrLf
    msg$ = msg$ + "FourCCckID =" & FourCCckID
    MsgBox msg$, vbOKOnly, "Invalid .wav file!"
    GetWaveTime = 0
    Close WaveFile
    Exit Function
End If
    
'Calculate file length in seconds
Length = (NumSamples / SampleRate) / NumChannels
Length = Length / (BitsPerSample / 8)

'Set the return value for the function
GetWaveTime = Length
    
    'Messages for debugging
    msg$ = "ckID1 =" & ckID1 & vbCrLf
    msg$ = msg$ + "RiffID =" & RIFFID & vbCrLf
    msg$ = msg$ + "ckID2 =" & ckID2 & vbCrLf
    msg$ = msg$ + "Filename =" & WaveFileName & vbCrLf
    msg$ = msg$ + "Format tag =" & Str$(FormatTag) & vbCrLf
    msg$ = msg$ + "Format Chunk Length =" & Str$(FormatChunkLength) & vbCrLf
    msg$ = msg$ + "Sample rate =" & Str$(SampleRate) & vbCrLf
    msg$ = msg$ + "Bits/Sample =" & Str$(BitsPerSample) & vbCrLf
    msg$ = msg$ + "Number of channels =" & Str$(NumChannels) & vbCrLf
    msg$ = msg$ + "Number of  samples =" & Str$(NumSamples) & vbCrLf
    msg$ = msg$ + "Length =" & Str$(Length) & " seconds."
    MsgBox msg$, vbOKOnly, "Wav file info..."

End Function

